package org.ninjasoft.rovr.net;
import java.net.*;
import java.io.*;
import java.util.Vector;
import java.util.Iterator;

public class Client {
	public static final int CONTROL_PORT = 5399;
	public static final int RESPONSE_PORT = 5398;
	
	public static final int COMMAND_UP = 8;
	public static final int COMMAND_DOWN = 2;
	public static final int COMMAND_LEFT = 4;
	public static final int COMMAND_RIGHT = 6;
	public static final int COMMAND_STOP = 5;
	
	private int moveFactor = 15;

	private String ipAddress = "127.0.0.1";
	private Socket commandSocket = null;
	private Socket responseSocket = null;
	private InputStream in = null;
	private OutputStream out = null;
	private Vector listeners = new Vector();
	private Thread eventThread = null;
	
	public Client() {
	}
	
	public Client(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public void connect() throws UnknownHostException, IOException {
		this.commandSocket = new Socket(this.ipAddress, CONTROL_PORT);
		this.responseSocket = new Socket(this.ipAddress, RESPONSE_PORT);
		this.out = commandSocket.getOutputStream();
		this.in = responseSocket.getInputStream();
		this.eventThread = new Thread(new ResponseListener());
		this.eventThread.start();
	}
	
	public void setMoveFactor(int i) {
		System.out.println("Speed " + i);
		if ((i >= 1) && (i <= 5))
			this.moveFactor = 5 * (i+2);
	}
	
	public int getMoveFactor() {
		return this.moveFactor;
	}
	
	public void sendCommand(int command) throws IOException {
		String commandString = "";
		switch(command){
		case COMMAND_UP:
			commandString = "[0," + (-1 * moveFactor) + "]\n";
			break;
		case COMMAND_DOWN:
			commandString = "[0," + moveFactor + "]\n";
			break;
		case COMMAND_LEFT:
			commandString = "[" + (-1 * moveFactor) + ",0]\n";
			break;
		case COMMAND_RIGHT:
			commandString = "[" + moveFactor + ",0]\n";
			break;
		case COMMAND_STOP:
			commandString = "[0,0]\n";
			break;
		}
		if (commandString.length() > 0) {
			out.write(commandString.getBytes());
		}
	}
	
	public void addMovementListener(MovementListener listener) {
		this.listeners.add(listener);
	}
	
	public void removeMovementListener(MovementListener listener) {
		this.listeners.remove(listener);
	}
	
	private void notifyListeners(int code) {
		for (Iterator i = this.listeners.iterator(); i.hasNext(); ) {
			((MovementListener) i.next()).movementResponse(code);
		}
	}
	
	public class ResponseListener implements Runnable {
		public void run() {
			try {
				while(true) {
					int ch = in.read();
					if (ch != -1)
						notifyListeners(ch);
				}
			}catch(IOException e){
				System.out.println("Remote closed connection");
			}
		}
	}
}
